//sketch for the ATTINY13A
#include <avr/io.h>
#include <EEPROM.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>

#define ticks_per_day 9852
#define ticks_per_minute 7

long unsigned int ticks=0;

#define nval 4
byte period_days; //number of days between on
byte ontime_mins; //number of 8s to remain on
byte ncycle; //number of times to switch on
byte istart; //index of cycle to start

int main(void)
{
  cli(); // Turn off global interrupts

  DDRB &= ~_BV(PB0); // Set PB0 as input for pushbutton
  PORTB |= _BV(PB0); // enable pull-up on PB0
  DDRB |= _BV(PB1); // Set PB1 as output for switching output
  DDRB |= _BV(PB2); // Set PB2 as output for flash-led

  //get input from the push-button on the program
  for (byte ival=0; ival<nval; ival++){
    //long light
    for(long unsigned int i=0; i<100000; i++)PORTB |= _BV(PB1); PORTB &= ~_BV(PB1);  

    //count the number of presses
    byte val=0;
    long unsigned int pushed=0;
    for(long unsigned int i=0; i<1000000; i++){
      if((PINB&_BV(PB0))==0)pushed++;
      else pushed=0;
      if(pushed==100)val++;
      if(pushed>100)PORTB |= _BV(PB1);
      else PORTB &= ~_BV(PB1);
    }

    //if no button pushed, use previous values stored in EEPROM
    if(val==0) val=EEPROM.read(ival);

    //give feedback on the number of flashes
    for(byte iflash=0; iflash<val; iflash++){
      for(long unsigned int i=0; i<10000; i++)PORTB |= _BV(PB1);  
      for(long unsigned int i=0; i<100000; i++)PORTB &= ~_BV(PB1);  
    }

    //interpret the input
    if(ival==0)period_days=val;
    if(ival==1)ontime_mins=val;  
    if(ival==2)ncycle=val;
    if(ival==3)istart=val;

    //store the values in the EEPROM
    EEPROM.write(ival,val);

  }

  wdt_reset(); // Reset watchdog (needed/wanted?)

  WDTCR = _BV(WDTIE) | _BV(WDP3) | _BV(WDP0); //8s

  sei(); // Turn on global interrupts
  
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  
  while(1)sleep_mode();   // go to sleep and wait for interrupt...  
}


// This will run every time the WDT causes an interrupt
ISR(WDT_vect)
{   
  // Shortly flash PB1
  PORTB |= _BV(PB1);  

  long unsigned int period_ticks=period_days*ticks_per_day;  
  long unsigned int icycle=ticks/period_ticks;
  long unsigned int itick=ticks%period_ticks;

  if(icycle>=(istart-1) and icycle<=ncycle+(istart-1) and itick<(ontime_mins*ticks_per_minute)){  
    PORTB |= _BV(PB2);
  } else{
    PORTB &= ~_BV(PB2);
  }
  ticks++;

  //optionally, insert delay to make longer flash
  //for(long unsigned int i=0; i<1000; i++)__asm__("nop\n\t"); 
  PORTB &= ~_BV(PB1);  

}


